//------------------------------------------------------------------------

//             Copyright (c) 1998 - 2001 Insight Electronics dba MemecCore

//------------------------------------------------------------------------

//                                       IMPORTANT

//       DO NOT REMOVE THIS NOTICE!  REMOVAL OF THIS NOTICE IS A VIOLATION OF YOUR 

//                       LICENSE AGREEMENT WITH INSIGHT AND MEMECCORE:

//

//     Use of this core is governed by a license agreement between Insight and the Licensee.  

//	   Use of this core by the Licensee for an unauthorized purpose, or by anyone other than 

//	   the Licensee, without the express written consent of Insight is an infringement of 

//	   Insight's intellectual property rights and may be enjoined by a court of law.   If you 

//	   are uncertain as to whether your intended use of this product is an authorized use, please 

//	   contact Insight at 1-888-845-5585. 

//

//

// FileName		: MC_UART.v

//		  		:

// Title		: UART Core

//              : 

// Description	: This is the top level of the core.  It instantiates

//              : the following modules:

//              : 	UART_BAUD

//              : 	UART_MODEM

//              : 	UART_INTR

//              : 	UART_XMIT

//              :   UART_RECV

//              :

// Functional   :

// Requirements : This module is a basic UART incorporating baud rate generator and modem interface sections.

//                It can be used as a 8250 reeplacement of by using the FIFO interface as a 16550 type.

//

// Description  

// of ports		

//  name      type    description

//	SIN        IN     Transmit Serial Data

//	TX_CE      IN     Transmit CLK enable

//	RX_CE      IN     Receive CLK enable

//	CTS_L      IN     Data Clear to Send	

//	DSR_L      IN     Data Set Ready

//	DCD_L      IN     Data Carrier Detect

//	RI_L       IN     Ring Indicator

//	DL	       IN     Delay Latch Register

//	LCR        IN     Line Control	Register

//	MCR        IN     Modem Control Register

//	IER        IN     Interrupt Enable Register

//	THR        IN     Transmit Holding Register

//	FMODE_RX   IN     Transmit FIFO Mode

//	FMODE_TX   IN     Receive FIFO Mode

//	THR_CE     IN     Transmit Holding Reg Clock Enable

//	FEMPTY     IN     FIFO Empty Flag

//	FFULL      IN     FIFO Full Flag

//	RBR_ACK    IN     Receive Buffer Register Acknowledge

//	IIR_ACK    IN     Interrupt Ident. Register Acknowledge

//	LSR_ACK    IN     Line Status Register Acknowledge

//	MSR_ACK    IN     Modem Status Register Acknowledge

//	CLK        IN     Master Clock

//	BRG_CE     IN     Master Clock enable

//	RST        IN     Master Reset (async)

//

//	SOUT       OUT    Receive Serial Data

//	BAUD_CE    OUT    Baud Clock Enable

//	RTS_L      OUT    Request to Send

//	DTR_L      OUT    Data Terminal Ready

//	OUT1_L     OUT    Output bit 1

//	OUT2_L     OUT    Output bit 2

//	RBR        OUT    Receive Buffer Register

//	LSR        OUT    Line Status Register

//	MSR        OUT    Modem Status Register

//	IIR        OUT    Interrupt Ident. Register

//	INTRPT     OUT    Interrupt Request

//	FREAD      OUT    FIFO Read Enable

//	FWRITE     OUT    FIFO Write Enable

//

// IncludeFiles	: None

//              : 

// Conventions	: Port names are 'UPPER' case.

//				: Internal wires and registers are 'lower' case.

//				: Parameters are first character 'Upper' case.

//				: Active low signals are identified with '_l' or '_L'

//				: appended to the wire, register, or port name.

//              : 

// Author		: DMeyers

//				:

// Company		: MemecCore

// 				: 7810 South Hardy Dr., Suite 104

//				: Tempe, Arizona 85284

//				: Phone (480)467-5000

//				: Fax   (480)467-5050

//				:

// ----------------------------------------------------------------------

// Revision History

// ----------------------------------------------------------------------

//  Version	| Ver.Date	| Author	| Description

//  1.0		| 3/15/99  	| DMeyers	| Baseline Release

// ----------------------------------------------------------------------

//  1.1		| 9/11/01  	| WJM       | Incorporated new header

// ----------------------------------------------------------------------



`timescale 1ns / 10ps



module XF_UART (

	// inputs

	SIN,			// Transmit Serial Data

	TX_CE,	 		// Transmit CLK enable

	RX_CE,	 		// Receive CLK enable

	CTS_L,			// Data Clear to Send	

	DSR_L,			// Data Set Ready

	DCD_L,		   	// Data Carrier Detect

	RI_L,			// Ring Indicator

	DL,				// Delay Latch Register

	LCR,			// Line Control	Register

	MCR,			// Modem Control Register

	IER,			// Interrupt Enable Register

	THR,			// Transmit Holding Register

	FMODE_RX,		// Transmit FIFO Mode

	FMODE_TX,		// Receive FIFO Mode

	THR_CE,			// Transmit Holding Reg Clock Enable

	FEMPTY,			// FIFO Empty Flag

	FFULL,			// FIFO Full Flag

	RBR_ACK,		// Receive Buffer Register Acknowledge

	IIR_ACK,		// Interrupt Ident. Register Acknowledge

	LSR_ACK,		// Line Status Register Acknowledge

	MSR_ACK,		// Modem Status Register Acknowledge

	CLK,	 		// Master Clock

	BRG_CE,			// Master Clock enable

	RST,			// Master Reset (async)

	// outputs

	SOUT,			// Receive Serial Data

	BAUD_CE,		// Baud Clock Enable

	RTS_L,			// Request to Send

	DTR_L,			// Data Terminal Ready

	OUT1_L,			// Output bit 1

	OUT2_L,			// Output bit 2

	RBR,			// Receive Buffer Register

	LSR,			// Line Status Register

	MSR,			// Modem Status Register

	IIR,			// Interrupt Ident. Register

	INTRPT,			// Interrupt Request

	FREAD,			// FIFO Read Enable

	FWRITE			// FIFO Write Enable

);

   // synthesis syn_edif_bit_format = "%u<%i>"

   // synthesis syn_edif_scalar_format = "%u"

   // synthesis syn_black_box



// inputs

input			SIN;

input			TX_CE;

input			RX_CE;

input			CTS_L;

input			DSR_L;

input			DCD_L;

input			RI_L;

input	[15:0]	DL;

input	[6:0]	LCR;

input	[4:0]	MCR;

input	[3:0]	IER;

input	[7:0]	THR;

input			FMODE_RX;

input			FMODE_TX;

input			THR_CE;

input			FEMPTY;

input			FFULL;

input			RBR_ACK;

input			IIR_ACK;

input			LSR_ACK;

input			MSR_ACK;

input			CLK;

input			BRG_CE;

input			RST;

// outputs

output			SOUT;

output			BAUD_CE;

output			RTS_L;

output			DTR_L;

output			OUT1_L;

output			OUT2_L;

output	[7:0]	RBR;

output	[6:0]	LSR;

output	[7:0]	MSR;

output	[2:0]	IIR;

output			INTRPT;

output			FREAD;

output			FWRITE;





endmodule





