----------------------------------------------------------------------------------------------------

--             :

-- File name   : LED_TB.VHD

--             :

-- Title       : PCI LED reference design testbench

--             :

-- IncludeFiles:

--             :

-- Created On  :

--             :

-- Comments    :

--             :

--             :

--             :

--             :

--             :

--             :

--             :

--             :

-- Conventions : - Port names are 'UPPER' case.

--             : - Internal signals and variables are 'lower' case.

--             : - Active low signals are identified with '_n' or '_N'.

--             : - constants begin with lower case 'c', rest upper case

--             : - states begin with lower case 's', rest upper case

--             : - aliases begin with lower case 'a', rest upper case

--             : - instances begin with lower case 'i', rest upper case

--             :

--             : Tab setting = 3

--             : Right Margin = 100

--             :

-- Limitations : For use in Xilinx FPGA

--             :

-- Developers  : Majid Kalsi

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--   Version No | Author    | Changes Made:                                            | Mod. Date:

--     v1.0     | mk        | First write                                              | 03/27/00

--     v2.0     | tc        | adapted for pci led ref design                           | 03/27/00

----------------------------------------------------------------------------------------------------





library IEEE;

use IEEE.STD_LOGIC_UNSIGNED.all;

use IEEE.STD_LOGIC_1164.all;

use IEEE.STD_LOGIC_ARITH.all;

USE IEEE.VITAL_PRIMITIVES.all;

USE IEEE.VITAL_TIMING.all;

use IEEE.STD_LOGIC_TEXTIO.all;

use STD.TEXTIO.all;

use WORK.UTILS_PKG.all;





entity LED_TB is

end LED_TB;





----------------------------------------------------------------------------------------------------

architecture TB_ARCHITECTURE of LED_TB is

----------------------------------------------------------------------------------------------------





----------------------------------------------------------------------------------------------------

-- Component definitions

----------------------------------------------------------------------------------------------------





component led_top

port (

   -- PCI CORE port definitions, do not modify

   PCLK            : in    std_logic;                     -- Please refer to PCI core documentation

   RST_N           : in    std_logic;                     -- Please refer to PCI core documentation

   REQ_N           : out   std_logic;                     -- Please refer to PCI core documentation

   GNT_N           : in    std_logic;                     -- Please refer to PCI core documentation

   IDSEL           : in    std_logic;                     -- Please refer to PCI core documentation

   AD              : inout std_logic_vector(31 downto 0); -- Please refer to PCI core documentation

   CBE             : inout std_logic_vector( 3 downto 0); -- Please refer to PCI core documentation

   FRAME_N         : inout std_logic;                     -- Please refer to PCI core documentation

   IRDY_N          : inout std_logic;                     -- Please refer to PCI core documentation

   DEVSEL_N        : inout std_logic;                     -- Please refer to PCI core documentation

   TRDY_N          : inout std_logic;                     -- Please refer to PCI core documentation

   STOP_N          : inout std_logic;                     -- Please refer to PCI core documentation

   PAR             : inout std_logic;                     -- Please refer to PCI core documentation

   PERR_N          : inout std_logic;                     -- Please refer to PCI core documentation

   SERR_N          : inout std_logic;                     -- Please refer to PCI core documentation

   INTR_A          : out   std_logic;                     -- Please refer to PCI core documentation

   --userapp i/o's 

   LED_TEST        : out std_logic;

   TENSDIGIT : out std_logic_vector(6 downto 0) ;

   ONESDIGIT : out std_logic_vector(6 downto 0) 

   

);

end component;





component PCI_Host

generic( LogFile : STRING  := "HOST.LOG");

port(

   -- command control interface

   INITIATOR_CMD  : in INTEGER;

   TARGET_CMD     : in INTEGER;

   END_SIM        : in BOOLEAN;

   INSERT_WAITS   : in boolean;

   CMD_ACK        : out boolean;

   BUSY           : out boolean;

   -- PCI signals

   CLK            : in     std_logic;

   RESET_N        : in     std_logic;

   INTR_A         : in     std_logic;

   REQ_N          : out    std_logic;

   GNT_N          : in     std_logic;

   FRAME_N        : inout  std_logic;

   IRDY_N         : inout  std_logic;

   IDSEL          : out    std_logic;

   DEVSEL_N       : inout  std_logic;

   TRDY_N         : inout  std_logic;

   STOP_N         : inout  std_logic;

   CBE            : inout  std_logic_vector(3 downto 0);

   AD             : inout  std_logic_vector(31 downto 0);

   PAR            : inout  std_logic

);

end component;





component PCI_arbiter

generic(

   SIZE        : positive := 2);

port (

   CLK      : in  std_logic;

   REQ_N    : in  std_logic_vector((SIZE-1) downto 0);

   GNT_N    : out std_logic_vector((SIZE-1) downto 0);

   FRAME_N  : in  std_logic;

   IRDY_N   : in  std_logic

);

end component;





----------------------------------------------------------------------------------------------------

-- define Types

----------------------------------------------------------------------------------------------------

type TARGET_TRANSFER_TYPE is (WAITE, NORMAL, RETRY, DWD, IDLE, ABORT);

type INITIATOR_TRANSFER_TYPE is (WAIT_BURST, WAIT_SINGLE, PROCEED, FINISH, IDLE);

subtype slv3 is std_logic_vector(2 downto 0);

subtype slv2 is std_logic_vector(1 downto 0);





----------------------------------------------------------------------------------------------------

-- define Constants

----------------------------------------------------------------------------------------------------

-- Host's Initiator commands

constant NONE                 : integer := 0;

constant LED_ON               : integer := 1;

constant LED_OFF              : integer := 2;



-- Host's Target command

constant TARGET_ABORT         : integer := 1;

constant MASTER_ABORT         : integer := 2;

constant PARITY_ERROR         : integer := 3;

constant PARTIAL_BURST        : integer := 4;

constant RETRY_BURST          : integer := 5;



----------------------------------------------------------------------------------------------------

-- define signals

----------------------------------------------------------------------------------------------------

-- PCI signals

signal ad        : std_logic_vector(31 downto 0);

signal cbe       : std_logic_vector( 3 downto 0);

signal par       : std_logic;

signal frame_n   : std_logic := 'H';

signal irdy_n    : std_logic := 'H';

signal trdy_n    : std_logic := 'H';

signal stop_n    : std_logic := 'H';

signal devsel_n  : std_logic := 'H';

signal idsel     : std_logic;

signal intr_a    : std_logic := 'H';

signal perr_n    : std_logic := 'H';

signal serr_n    : std_logic := 'H';

signal req_n      : std_logic_vector(1 downto 0) := (others => 'H');

signal gnt_n      : std_logic_vector(1 downto 0) := (others => 'H'); 

signal rst_n     : std_logic := 'H';

signal pclk      : std_logic;

signal req64_n   : std_logic := 'H';

signal ack64_n   : std_logic := 'H';

signal par64     : std_logic;

signal target_transfer_state : TARGET_TRANSFER_TYPE;

signal initiator_transfer_state : INITIATOR_TRANSFER_TYPE;



-- PCI Host signals

signal initiator_cmd : integer;

signal target_cmd    : integer;

signal insert_waits  : boolean;

signal cmd_ack       : boolean;

signal host_busy     : boolean;



-- misc signals

signal end_sim : boolean := false;

signal gnd : std_logic;

signal vcc : std_logic;



signal led_test : std_logic ;

signal tensdigit : std_logic_vector(6 downto 0) ;

signal onesdigit : std_logic_vector(6 downto 0) ;



----------------------------------------------------------------------------------------------------

begin    -- begin Architecture

----------------------------------------------------------------------------------------------------





-- assign constant values

gnd <= '0';

vcc <= '1';





----------------------------------------------------------------------------------------------------

-- instantiate components

----------------------------------------------------------------------------------------------------

UUT : led_top

port map(

   PCLK            => pclk,

   RST_N           => rst_n,

   REQ_N           => open,

   GNT_N           => gnt_n(0),

   IDSEL           => idsel,

   FRAME_N         => frame_n,

   IRDY_N          => irdy_n,

   DEVSEL_N        => devsel_n,

   TRDY_N          => trdy_n,

   STOP_N          => stop_n,

   AD              => ad,

   CBE             => cbe,

   PAR             => par,

   PERR_N          => perr_n,

   SERR_N          => serr_n,

   INTR_A          => intr_a,

   LED_TEST        => led_test,

   TENSDIGIT       => tensdigit,

   ONESDIGIT       => onesdigit

);





ARBITER : PCI_arbiter

generic map (

   SIZE     => 2)

port map(

   CLK      => pclk,

   FRAME_N  => frame_n,

   IRDY_N   => irdy_n,

   REQ_N   => req_n,     -- LED request

   GNT_N   => gnt_n      -- LED grant

);



HOST : PCI_Host

generic map(LogFile => ".\HOST.LOG")

port map(

   INITIATOR_CMD    => initiator_cmd,

   TARGET_CMD       => target_cmd,

   END_SIM          => end_sim,

   INSERT_WAITS     => insert_waits,

   CMD_ACK          => cmd_ack,

   BUSY             => host_busy,

   CLK              => pclk,

   RESET_N          => rst_n,

   INTR_A           => intr_a,

   REQ_N           => req_n(1),

   GNT_N           => gnt_n(1),

   FRAME_N          => frame_n,

   IRDY_N           => irdy_n,

   IDSEL            => idsel,

   DEVSEL_N         => devsel_n,

   TRDY_N           => trdy_n,

   STOP_N           => stop_n,

   CBE              => cbe(3 downto 0),

   AD               => ad(31 downto 0),

   PAR              => par

);





-- Pull ups on PCI bus

devsel_n    <= 'H';

frame_n     <= 'H';

irdy_n      <= 'H';

par         <= 'Z';

perr_n      <= 'H';

serr_n      <= 'H';

stop_n      <= 'H';

trdy_n      <= 'H';

req_n(0)      <= 'H';        -- LED request

gnt_n(0)      <= 'H';        -- LED grant

req_n(1)      <= 'H';        -- host request

gnt_n(1)      <= 'H';        -- host grant

intr_a      <= 'H';

req64_n     <= 'H';

ack64_n     <= 'H';

par64       <= 'Z';



----------------------------------------------------------------------------------------------------

-- Main Stimulus Process

----------------------------------------------------------------------------------------------------

STIMULUS : process



----------------------------------------------------------------------------------------------------

-- Define Procedures

----------------------------------------------------------------------------------------------------

variable inc : integer ;

variable dec : integer ;





----------------------------------------------------------------------------------------------------

begin   -- Begin Main Stimulus Process

----------------------------------------------------------------------------------------------------

   inc := 0;

   dec := 0;

   end_sim <= false;

   insert_waits <= false;

   initiator_cmd <= NONE;

   target_cmd <= NONE;

   rst_n <= '0';

   wait for 115 ns;

   rst_n <= '1';



   -- wait for initialization to finish

   if host_busy then

      wait until not host_busy;

   end if;



-----------------------------------------------

-- Turn the LED on and off

-----------------------------------------------

   initiator_cmd <= LED_OFF;

   wait until cmd_ack;

   initiator_cmd <= NONE;



   while inc < 110 loop

      initiator_cmd <= LED_ON;

      wait until cmd_ack;

      initiator_cmd <= NONE;

      inc := inc + 1 ;

   end loop;

   

   while dec < 110 loop

      initiator_cmd <= LED_OFF;

      wait until cmd_ack;

      initiator_cmd <= NONE;

      dec := dec + 1 ;

   end loop;



   initiator_cmd <= LED_ON;

   wait until cmd_ack;

   initiator_cmd <= NONE;



   -- End simulation Control

   wait for 500 ns;

   if host_busy then

      wait until not(host_busy);

   end if;

   end_sim <= true;

   assert(false)

   report "End of Simulation"

   severity failure;



----------------------------------------------------------------------------------------------------

end process;  -- End Main Stimulus Process

----------------------------------------------------------------------------------------------------





----------------------------------------------------------------------------------------------------

--  PCI Transfer State

----------------------------------------------------------------------------------------------------

PCI_BUS_STATE : process(trdy_n, devsel_n, stop_n, irdy_n, frame_n)

begin

   case slv3'(trdy_n, devsel_n, stop_n) is

      when "101" =>

            target_transfer_state <= WAITE;



      when "001" =>

         target_transfer_state <= NORMAL;



      when "100" =>

         target_transfer_state <= RETRY;



      when "000" =>

         target_transfer_state <= DWD; -- Disconnect with data



      when "110" =>

         target_transfer_state <= ABORT;



      when others =>

         target_transfer_state <= IDLE;

   end case;



   case slv2'(irdy_n, frame_n) is

      when "10"  => initiator_transfer_state <= WAIT_BURST;

      when "11"  => initiator_transfer_state <= WAIT_SINGLE;

      when "00"  => initiator_transfer_state <= PROCEED;

      when "01"  => initiator_transfer_state <= FINISH;

      when others => initiator_transfer_state <= IDLE;

   end case;

end process;





----------------------------------------------------------------------------------------------------

-- PCLK,  33 MHz clock

----------------------------------------------------------------------------------------------------

PCLK_GEN : process

begin

   if end_sim = false then

      pclk <= '0';

      wait for 15 ns;

   else

      wait;

   end if;

   if end_sim = false then

      pclk <= '1';

      wait for 15 ns;

   else

      wait;

   end if;

end process;





----------------------------------------------------------------------------------------------------

end TB_ARCHITECTURE;

----------------------------------------------------------------------------------------------------



----------------------------------------------------------------------------------------------------

-- Configuration Stuff

----------------------------------------------------------------------------------------------------

--configuration TESTBENCH_FOR_LED of LED_TB is

-- for TB_ARCHITECTURE

--    for UUT : LED_TOP

--       use entity work.LED_TOP(STRUCTURE);

--    end for;

-- end for;

--end TESTBENCH_FOR_LED;

----------------------------------------------------------------------------------------------------

--

----------------------------------------------------------------------------------------------------



