/***********************************************************************

  File:   dumb_arbiter.v

  This is a functional simulation model for a simple arbiter.  This
  is not synthesizable.  This file is only for simulation.


***********************************************************************/

`timescale 1 ns/1 ps

module DUMB_ARBITER
(
   CLK,
   REQ_N,
   GNT_N,
   FRAME_N,
   IRDY_N
);

parameter pSIZE = 3;          // maximum 8
parameter pTGNT = 5;

input  CLK;
input  FRAME_N;
input  IRDY_N;
input  [(pSIZE-1) : 0] REQ_N;
output [(pSIZE-1) : 0] GNT_N;

reg [(pSIZE-1) : 0] last_time;
reg [(pSIZE-1) : 0] grant_n;
wire granted;


assign granted = (|(~grant_n)) ? 1 : 0;

initial begin
   last_time = 'b0;
	grant_n = 'hFF;
end

always @(posedge CLK)
begin : ARB
   integer i, j;

   // if request active
	if(&(REQ_N) == 0) begin

      // figure which was granted last time
      i = 0;
      while((i < pSIZE) && (last_time[i] == 0))
         i = i + 1;

      // if none were granted, ie reset condition
      // or if rollover
      j = ((i == pSIZE) || (i == (pSIZE-1))) ? 0 : i + 1;

      // find requesting device
      while(REQ_N[j] == 1) begin
         j = (j == (pSIZE-1)) ? 0 : j+1;
      end

      // give GRANT to requesting device
      grant_n[j] = 1'b0;
      last_time = 'b0;
      last_time[j]  = 1'b1;

      // wait for frame to assert
      while(FRAME_N == 1)
         @(posedge CLK);

      // remove grant
		grant_n = 'hFF;

      // wait for IRDY_N to assert
      while(IRDY_N == 1)
         @(posedge CLK);

      // wait for IRDY_N to de-assert
      while(IRDY_N == 0)
         @(posedge CLK);
   end
end

assign #pTGNT GNT_N = grant_n;


endmodule
