//--------------------------------------------------------------------------------------------------

//             :

// File name   : PCI_DEVICE_TARGET.V

//             :

// Title       : Generic PCI device target Tasks

//             :

// Comments    :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             : - functions begin with lower case 'f', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | mk        | First write                                              | 00/00/00

//--------------------------------------------------------------------------------------------------







//--------------------------------------------------------------------------------------------------

task tSERVICE_CFG_REQUEST;

//--------------------------------------------------------------------------------------------------



	integer count;

   reg read_access;

   reg [7:0] addr;

   reg [31:0] data;



   begin

	read_access = (CBE[3:0] == `CFG_RD);

   addr = AD[7:0];



	TBUSY = `TRUE;

   TSTATE = "DOING CFG REQUEST";



   // wait DEVSEL initial latency, then drive TRDY

   count = 0;

   while(count != pDECODE) begin

       @(posedge CLK);

       count = count + 1;

   end

   reg_devsel_n = 1'b0;

   reg_trdy_n = 1'b1;

   reg_stop_n = 1'b1;

   targ_oe = 1'b1;



   // wait TRDY initial latency

   count = 0;

   while(count != TRDY_INITIAL_LATENCY) begin

       @(posedge CLK);

       count = count + 1;

   end

   reg_trdy_n = 1'b0;

   reg_stop_n = 1'b0;   // disconnect with data on first data phase



	if(read_access) begin

      // drive data onto bus

      ad_oe = 1'b1;

      case(addr)

         8'h00 : reg_ad = {pDEVICE_ID, pVENDOR_ID};

         8'h04 : reg_ad = {cfg_stat_reg, cfg_cmd_reg};

         8'h08 : reg_ad = {pCLASS_CODE, pREV_ID};

         8'h0C : reg_ad = {bist, pHEADER_TYPE, lat_timer_reg, line_size_reg};

         8'h10 : reg_ad = bars[0];

         8'h14 : reg_ad = bars[1];

         8'h18 : reg_ad = bars[2];

         8'h1C : reg_ad = bars[3];

         8'h20 : reg_ad = bars[4];

         8'h24 : reg_ad = bars[5];

         8'h28 : reg_ad = 32'h0000_0000;        // CardBus CIS pointer not supported

         8'h2C : reg_ad = {pSUB_ID, pSUB_VID};

         8'h30 : reg_ad = 32'h0000_0000;        // Expansion ROM not supported

         8'h34 : reg_ad = 32'h0000_0000;        // Capabilities Pointer not supported

         8'h38 : reg_ad = 32'h0000_0000;        // Reserved

         8'h3C : reg_ad = {max_lat_reg, min_gnt_reg, pINT_PIN, int_line_reg};

         default : reg_ad = 32'hXXXX_XXXX;

      endcase

   end

   

   // Check IRDY_N

   if(IRDY_N)

   	while(IRDY_N == 1)

      	@(posedge CLK);

   else

     	@(posedge CLK);



	if(!read_access) begin

      case(addr)

         8'h04 :

				begin

               if(CBE[0] == 0) cfg_cmd_reg[7:0] = AD[7:0] & cfg_cmd_mask[7:0];

               if(CBE[1] == 0) cfg_cmd_reg[15:8] = AD[15:8] & cfg_cmd_mask[15:8];

         	end



         8'h0C :

         	begin

               if(CBE[0] == 0) line_size_reg = AD[7:0];

               if(CBE[1] == 0) lat_timer_reg = AD[15:8];

               if((CBE[3] == 0) && (pBIST_EN)) bist_reg = AD[30:24];

         	end



         8'h10 :  tWRITE_BAR(addr);

			8'h14 :  tWRITE_BAR(addr);

			8'h18 :  tWRITE_BAR(addr);

			8'h1C :  tWRITE_BAR(addr);

			8'h20 :  tWRITE_BAR(addr);

			8'h24 :  tWRITE_BAR(addr);



         8'h3C :

				begin

               if(CBE[0] == 0) int_line_reg = AD[7:0];

               if(CBE[2] == 0) min_gnt_reg = AD[23:16];

               if(CBE[3] == 0) max_lat_reg = AD[31:24];

            end

         default : 		;// do nothing

      endcase

   end



	// de-assert target signals

   ad_oe = 1'b0;

   reg_devsel_n = 1'b1;

   reg_trdy_n = 1'b1;

   reg_stop_n = 1'b1;

   targ_oe = 1'b0;



	TBUSY = `FALSE;

   TSTATE = "IDLE";



	end

endtask





//--------------------------------------------------------------------------------------------------

task tWRITE_BAR;

//--------------------------------------------------------------------------------------------------



	// input/output declarations

	input [7:0] addr;



   // local variables

   reg [31:0] data;

   reg [31:0] mask;



   begin

      data = bars[addr[7:2] - 4];

      mask = bar_masks[addr[7:2] - 4];



      if(CBE[0] == 0) data[ 7: 0] = AD[ 7: 0] & mask[ 7: 0];

      if(CBE[1] == 0) data[15: 8] = AD[15: 8] & mask[15: 8];

      if(CBE[2] == 0) data[23:16] = AD[23:16] & mask[23:16];

      if(CBE[3] == 0) data[31:24] = AD[31:24] & mask[31:24];



      bars[addr[7:2] - 4] = data;

   end

endtask







//--------------------------------------------------------------------------------------------------

task tSERVICE_INT_ACK_REQUEST;

//--------------------------------------------------------------------------------------------------



   begin



   //----------------------------

   //

   //  Add code here

   //

   //----------------------------



   end

endtask







//--------------------------------------------------------------------------------------------------

task tSERVICE_SPECIAL_REQUEST;

//--------------------------------------------------------------------------------------------------



   begin



   //----------------------------

   //

   //  Add code here

   //

   //----------------------------



   end

endtask







//--------------------------------------------------------------------------------------------------

task tSERVICE_DAC_REQUEST;

//--------------------------------------------------------------------------------------------------



   begin



   //----------------------------

   //

   //  Add code here

   //

   //----------------------------



   end

endtask







//--------------------------------------------------------------------------------------------------

task tSERVICE_TARGET_REQUEST;

//--------------------------------------------------------------------------------------------------



   // local variables

	integer index;             			// index into ram

   integer count;             			// count of the # of data phases

   integer data_cnt;    					// count of the # of valid data phases, ie not wait states

   integer i;                          // loop variable

   reg access64bit;                		// 64 bit access flag

   reg     write_access;      			// write access flag

   reg     last_data;                  // last data flag

   reg [3:0] access_type;              // type of PCI access

   reg [(20*8-1):0] str_access_type; 	// String describing type of access

   integer first_phase;                // first_phase flag

   reg [3:0] term_type;                // termination type

   reg [3:0] trdy_delay;               // TRDY Latency

   reg [3:0] wait_delay;               // Wait state duration

   reg wait_state;                     // wait state flag



	begin

		TBUSY = `TRUE;

	   TSTATE = "DOING TARGET REQUEST";

      first_phase = 0;



	   // if performing master abort then do nothing

		if(TCMD == `MABORT)

	      tINTERCYCLE_GAP;

	   else begin

         // if TCMD is Random, determine termination type

         if(TCMD == `RANDOM) begin

            // 34 % probability of normal termination

            // 33 % probability of DWD termination

            // 33 % probability of DWOD termination

            if(random_num <= 33.0)

               term_type = `DWOD;

            else if(random_num <= 66.0)

               term_type = `DWD;

            else

               term_type = `NONE;

         end

         else

            term_type = TCMD;





	      // Determine if 64 bit access

	      if(!REQ64_N)

	         access64bit = 1;   // Modify for test

            //access64bit = 0;

	      else

	         access64bit = 0;



	      // Determine index into RAM

	      if(access64bit)

	         index = AD[(pMEM_BITS+1) : 3];

	      else

	      	index = AD[(pMEM_BITS+1) : 2];



	      // Determine if write access

			write_access = (CBE[3:0] ==`IO_WR) || (CBE[3:0] == `MEM_WR) || (CBE[3:0] == `MWI);

	      access_type = CBE[3:0];

	      str_access_type = fGET_TYPE_STRING(CBE[3:0]);



	      // wait DEVSEL initial latency

	      count = 0;

	      while(count !== pDECODE) begin

	          @(posedge CLK);

	          count = count + 1;

	      end



         // drive target signals

	      reg_devsel_n = 1'b0;

	      reg_ack64_n = !access64bit;

	      reg_trdy_n = 1'b1;

	      reg_stop_n = 1'b1;

	      targ_oe = 1'b1;

	   	serr_oe = 1'b1;



         // if TCMD is Random, determine TRDY initial latency

         if(TCMD == `RANDOM) begin

            // 25 % probability of 0 cycles of latency

            // 25 % probability of 1 cycles of latency

            // 25 % probability of 2 cycles of latency

            // 25 % probability of 3 cycles of latency

            if(random_num <= 25.0)

               trdy_delay = 0;

            else if(random_num <= 50.0)

               trdy_delay = 1;

            else if(random_num <= 75.0)

               trdy_delay = 2;

            else

               trdy_delay = 3;

         end

         else

            trdy_delay = TRDY_INITIAL_LATENCY;



	      // wait TRDY initial latency, then drive TRDY

	      count = 0;

	      while(count !== trdy_delay) begin

	          @(posedge CLK);

	          count = count + 1;

	      end



	      // Determine end of burst value

	      // If full transfer then ignore TERM_CNT parameter, otherwise

	      // check if doing retry.  This is so we can toggle between a

			// terminal count of 0 and 1, otherwise assign TERM_CNT

         case(TCMD)

            `NONE:   

					burst_end = 1000;

            

				`DWOD:

               if(TERM_CNT == 0) begin

                  if(burst_end == 0) begin

                     if(random_num <=  33.3)

                     	burst_end = 1;

                     else if(random_num <=  66.7)

                        burst_end = 1;

                     else

                        burst_end = 1;

                  end

                  else

                     burst_end = 0;

               end

               else

                  burst_end = TERM_CNT;



            `RANDOM:

               case(term_type)

                  `NONE:

							burst_end = 1000;



                  `DWD:

            			// 20 % probability of terminating after 1 cycle

            			// 20 % probability of terminating after 2 cycles

            			// 20 % probability of terminating after 3 cycles

            			// 20 % probability of terminating after 4 cycles

            			// 20 % probability of terminating after 5 cycles

                     if(random_num <= 20.0)

                        burst_end = 1;

                     else if(random_num <= 40.0)

                        burst_end = 2;

                     else if(random_num <= 60.0)

                        burst_end = 3;

                     else if(random_num <= 80.0)

                        burst_end = 4;

                     else

                        burst_end = 5;



                  `DWOD:

            			// 17 % probability of terminating after 0 cycle

            			// 17 % probability of terminating after 1 cycles

            			// 17 % probability of terminating after 2 cycles

            			// 17 % probability of terminating after 3 cycles

            			// 17 % probability of terminating after 4 cycles

            			// 15 % probability of terminating after 5 cycles

                     if(random_num <= 17.0)

                        burst_end = 0;

                     else if(random_num <= 34.0)

                        burst_end = 1;

                     else if(random_num <= 51.0)

                        burst_end = 2;

                     else if(random_num <= 68.0)

                        burst_end = 3;

                     else if(random_num <= 85.0)

                        burst_end = 4;

                     else

                        burst_end = 5;



                  default: burst_end = 1000;

               endcase



            default:

               burst_end = TERM_CNT;

         endcase





	      // initialize counters/flags

	      data_cnt = 0;

         wait_state = 1;



			if(TCMD == `RANDOM)

            count = 0;

         else

	      	count = INITIAL_VALUE;

	      last_data = 0;



			if((pDECODE == `FAST) && (trdy_delay == 0))

         	first_phase = 1;



         // loop when:

         // (1) not end of burst and

         // (2.a) FRAME_N = 0 or

         // (2.b) FRAME_N = 1 and last data phase hasn't been accessed

         while((data_cnt < burst_end) && ((!FRAME_N) || (FRAME_N && !last_data))) begin

            // Check if wait needs to be inserted

            if((INSERT_WAITS && (count % MOD_AMT == RESULT)) ||

               ((TCMD == `RANDOM) && (random_num <= 60.0) && !wait_state)) begin



               // determine duration of wait state

               if(TCMD == `RANDOM) begin

                  if(random_num <= 30.0)

                     wait_delay = 1;

                  else if(random_num <= 50.0)

                     wait_delay = 2;

                  else

                     wait_delay = 3;



                  // set flag

                  wait_state = 1;

               end

               else

                  wait_delay = DURATION;



               // insert wait state, length of WAIT_DELAY

               reg_trdy_n = 1'b1;

               first_phase = 0;

               for(i = 0; i < wait_delay; i = i + 1) begin

                  @(posedge CLK);

                  $display($time, " ns [%s TARGET]  Wait inserted", pINST_NAME);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  Wait inserted", pINST_NAME);

               end

            end

            // if not wait state, transfer data

            else begin

               // clear flag

               wait_state = 0;



               // insert data phase

               reg_trdy_n = 1'b0;



               // if disconnect with data, then drive STOP_N

               if((term_type == `DWD) && (data_cnt == (burst_end - 1)) && !FRAME_N)

                  reg_stop_n = 1'b0;



               // if READ access

               if(!write_access) begin

                  // read access, drive AD

                  ad_oe = 1;



                  // check which BAR

                  if(bar_hit[0]) reg_ad = ram0[index];

                  if(bar_hit[1]) reg_ad = ram1[index];

                  if(bar_hit[2]) reg_ad = ram2[index];

                  if(bar_hit[3]) reg_ad = ram3[index];

                  if(bar_hit[4]) reg_ad = ram4[index];

                  if(bar_hit[5]) reg_ad = ram5[index];



                  // wait prop delay

                  #pTDEL;

               end



               // Check IRDY, ie initiator waits

               while(IRDY_N)

                  @(posedge CLK);



               // if WRITE access

               if(write_access) begin : WR_ACCESS

                  reg [63:0] temp;



                  // wait prop delay

                  if(!first_phase)

                     #pTDEL;



                  // check which BAR

                  if(bar_hit[0]) temp = ram0[index];

                  if(bar_hit[1]) temp = ram1[index];

                  if(bar_hit[2]) temp = ram2[index];

                  if(bar_hit[3]) temp = ram3[index];

                  if(bar_hit[4]) temp = ram4[index];

                  if(bar_hit[5]) temp = ram5[index];



                  if(CBE[0] == 0) temp[ 7: 0] = AD[ 7: 0];

                  if(CBE[1] == 0) temp[15: 8] = AD[15: 8];

                  if(CBE[2] == 0) temp[23:16] = AD[23:16];

                  if(CBE[3] == 0) temp[31:24] = AD[31:24];



/*                  if(access64bit)   begin

                     if(CBE[4] == 0) temp[39:32] = AD[39:32];

                     if(CBE[5] == 0) temp[47:40] = AD[47:40];

                     if(CBE[6] == 0) temp[55:48] = AD[55:48];

                     if(CBE[7] == 0) temp[63:56] = AD[63:56];

                  	end   */



                  // write ram

                  if(bar_hit[0]) ram0[index] = temp;

                  if(bar_hit[1]) ram1[index] = temp;

                  if(bar_hit[2]) ram2[index] = temp;

                  if(bar_hit[3]) ram3[index] = temp;

                  if(bar_hit[4]) ram4[index] = temp;

                  if(bar_hit[5]) ram5[index] = temp;



               end



   /*            if(access64bit) begin

                  $display($time, " ns [%s TARGET]  %S ACCESS, CBE: %B  AD: %16HH ", pINST_NAME, str_access_type, CBE[7:0], AD[63:0]);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  %S ACCESS, CBE: %B  AD: %16HH ", pINST_NAME, str_access_type, CBE[7:0], AD[63:0]);

               end */

               else begin

                  $display($time, " ns [%s TARGET]  %S ACCESS, CBE: %B  AD: %8HH ", pINST_NAME, str_access_type, CBE[3:0], AD[31:0]);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  %S ACCESS, CBE: %B  AD: %8HH ", pINST_NAME, str_access_type, CBE[3:0], AD[31:0]);

               end



               // wait for clk edge

               if(first_phase)

                  first_phase = 0;

               else

                  @(posedge CLK);



               // update last data flag

               if(FRAME_N == 1)

                  last_data = 1;



               // increment counters

               index = index + 1;            // increment to next RAM location

               data_cnt = data_cnt + 1;      // increment valid data phase counter

            end

            count = count + 1;

         end



         // Terminate the access:

         // if BURST_END reached, then terminate as defined by TERM_TYPE

         // otherwise terminate with "normal" termination

         if(data_cnt == burst_end) begin

            case(term_type)

               `TABORT: begin

                  // terminate with Target Abort

                  reg_devsel_n = 1'b1;

                  reg_ack64_n = 1'b1;

                  reg_trdy_n = 1'b1;

                  reg_stop_n = 1'b0;

                  ad_oe = 1'b0;



                  $display($time, " ns [%s TARGET]  Terminating access with Target Abort!", pINST_NAME);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  Terminating access with Target Abort!", pINST_NAME);



                  @(posedge CLK);

                  @(posedge CLK);

               end



               `DWD: begin

                  // terminate with "disconnect with data"

                  reg_trdy_n <= 1'b1;

                  ad_oe <= 1'b0;



                  if(!STOP_N) begin

                     $display($time, " ns [%s TARGET]  Terminating access with DWD!", pINST_NAME);

                     $fdisplay(logfile, $time, " ns [%s TARGET]  Terminating access with DWD!", pINST_NAME);

                     @(posedge CLK);

                  end

               end



               `DWOD: begin

                  // terminate with "disconnect without data"

                  reg_devsel_n = 1'b0;

                  reg_trdy_n = 1'b1;

                  reg_stop_n = 1'b0;

                  ad_oe <= 1'b0;



                  $display($time, " ns [%s TARGET]  Terminating access with DWOD!", pINST_NAME);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  Terminating access with DWOD!", pINST_NAME);



                  @(posedge CLK);

                  @(posedge CLK);

               end



               `SYS_ERR: begin

                  reg_serr_n = 1'b0;

                  @(posedge CLK);

               end



               `PAR_ERR: begin

                  insert_perr = 1'b1;

                  @(posedge CLK);

                  insert_perr = 1'b0;

               end



               default: begin

                  $display($time, " ns [%s TARGET]  Terminating access with UNKNOWN termination!", pINST_NAME);

                  $fdisplay(logfile, $time, " ns [%s TARGET]  Terminating access with UNKNOWN termination!", pINST_NAME);

               end

            endcase

         end



	      // de-assert signals

	      reg_devsel_n = 1'b1;

	      reg_ack64_n = 1'b1;

	      reg_trdy_n = 1'b1;

	      reg_stop_n = 1'b1;

	      reg_serr_n <= 1'b1;

	      ad_oe <= 1'b0;

	   	serr_oe = 1'b0;

	     	@(posedge CLK);



	      // disable driving of target signals

	      targ_oe <= 1'b0;

		end





		TBUSY = `FALSE;

	   TSTATE = "IDLE";

	end

endtask





function [(20*8-1):0] fGET_TYPE_STRING;

	input [3:0] cbe;



   begin

      case(cbe)

			`INT_ACK : fGET_TYPE_STRING = "INT_ACK";

         `SPECIAL : fGET_TYPE_STRING = "SPECIAL";

         `IO_RD   : fGET_TYPE_STRING = "IO_RD";

         `IO_WR   : fGET_TYPE_STRING = "IO_WR";

         `MEM_RD  : fGET_TYPE_STRING = "MEM_RD";

         `MEM_WR  : fGET_TYPE_STRING = "MEM_WR";

         `CFG_RD  : fGET_TYPE_STRING = "CFG_RD";

         `CFG_WR  : fGET_TYPE_STRING = "CFG_WR";

         `MRM     : fGET_TYPE_STRING = "MRM";

         `DAC     : fGET_TYPE_STRING = "DAC";

         `MRL     : fGET_TYPE_STRING = "MRL";

         `MWI     : fGET_TYPE_STRING = "MWI";

         default  : fGET_TYPE_STRING = "???";

      endcase

   end

endfunction













