//--------------------------------------------------------------------------------------------------

//             :

// File name   : SDRAM_TB_CONTROL.V

//             :

// Title       : Controller for the S2PCI SDRAM testbench

//             :

// Comments    :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | da        | First Written          												| 27/08/01

//--------------------------------------------------------------------------------------------------





`timescale 1 ns/1 ps

`include "PCI_device_defines.v"





module SDRAM_TB_CONTROL

   (

      // PCI control signals

    pci_ICMD,

   	pci_ICMD_ACK,

   	pci_ISTATE,

   	pci_IBUSY,

   	pci_TCMD,

   	pci_TCMD_ACK,

   	pci_TSTATE,

   	pci_TBUSY,



    pci_RST_N,



    pci_CLK,

   	// PCI Initial Latency Control

   	pci_IRDY_INITIAL_LATENCY,

   	pci_TRDY_INITIAL_LATENCY,



   	// PCI Wait State Control

   	pci_INSERT_WAITS,

   	pci_INITIAL_VALUE,

   	pci_TERM_CNT,

   	pci_MOD_AMT,

   	pci_RESULT,

   	pci_DURATION,

	

      // General Testbench Control Signals



      END_SIM

   );







/////////////////////////////////////////////////////////////////////////////////////////////

// Port Declerations

////////////////////////////////////////////////////////////////////////////////////////////

output [3:0] 	pci_ICMD;

input 			pci_ICMD_ACK;

input  [99:0]	pci_ISTATE;

input		 	pci_IBUSY;

output [3:0] 	pci_TCMD;

input 			pci_TCMD_ACK;

input  [99:0]	pci_TSTATE;

input 			pci_TBUSY;



output         	pci_RST_N;

input          	pci_CLK;



// Initial Latency Control

output [3:0]	pci_IRDY_INITIAL_LATENCY;

output [3:0]	pci_TRDY_INITIAL_LATENCY;



// Wait State Control, see description above

output 			pci_INSERT_WAITS;

output [3:0] 	pci_INITIAL_VALUE;

output [3:0] 	pci_TERM_CNT;

output [3:0] 	pci_MOD_AMT;

output [3:0] 	pci_RESULT;

output [3:0] 	pci_DURATION;





output END_SIM;



reg [3:0] 	pci_ICMD;

reg [3:0] 	pci_TCMD;

reg [3:0]   pci_IRDY_INITIAL_LATENCY;

reg [3:0]   pci_TRDY_INITIAL_LATENCY;

reg 			pci_INSERT_WAITS;

reg [3:0] 	pci_INITIAL_VALUE;

reg [3:0] 	pci_TERM_CNT;

reg [3:0] 	pci_MOD_AMT;

reg [3:0] 	pci_RESULT;

reg [3:0] 	pci_DURATION;



reg         pci_RST_N;

reg         END_SIM;



integer file_ptr;



// Set up the signals

initial

begin : INIT_BLK

   pci_ICMD = `NONE;

   pci_TCMD = `NONE;

   pci_IRDY_INITIAL_LATENCY = 0;

   pci_TRDY_INITIAL_LATENCY = 1;

   pci_INSERT_WAITS = `FALSE;

   pci_TERM_CNT = 5;

   pci_MOD_AMT  = 4;

   pci_RESULT   = 3;

   pci_INITIAL_VALUE = 2;

   pci_DURATION = 1;



	END_SIM = 0;



   pci_RST_N = 1'b0;

   file_ptr = $fopen("CONTROL_LOG.tbl");

end





//-----------------------------------------------

//  Main Stimulus Block

//-----------------------------------------------

always begin

   $fdisplay(file_ptr, $time, "ns Beginning Simulation");

   $display($time, "ns Beginning Simulation");

   #115;

   pci_RST_N = 1'b1;

   $fdisplay (file_ptr, $time, "ns Coming out of Reset");

   $display ($time, "ns Coming out of Reset");

   wait (pci_IBUSY);

   $fdisplay (file_ptr, $time, " ns Beginning PCI configuration");

   $display ($time, " ns Beginning PCI configuration");



   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns PCI Configuration Complete");

   $display ($time, "ns PCI Configuration Complete");





   #3000





    tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;



   	tWAIT_4CLKS;





   		pci_ICMD = `TEST1;   // Write a single burst to the FIFO and read it back



	@(posedge pci_ICMD_ACK)

	$display ($time, "ns Test 1 -- Write a single burst to the FIFO and read it back");

   	$fdisplay (file_ptr, $time, "ns Test 1 -- Write a single burst to the FIFO and read it back");

   	wait (pci_IBUSY);

   	pci_ICMD = `NONE;

   	wait (!pci_IBUSY);

   	$fdisplay (file_ptr, $time, "ns Finished test 1");

	$display ($time, "ns Finished test 1");



      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;



	pci_ICMD = `TEST2;   // Multi size bursts



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 2 -- Variable Burst Size Test");

   $fdisplay (file_ptr, $time, " ns, Test 2 -- Variable Burst Size Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 2");

	$display ($time, "ns Finished test 2");



      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;



	pci_ICMD = `TEST3;   // Wait state test



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 3 -- Wait state Test");

   $fdisplay (file_ptr, $time, " ns, Test 3 -- Wait state Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 3");

	$display ($time, "ns Finished test 3");



      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;



	pci_ICMD = `TEST4;   //  Byte Enable test



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 4 -- Byte Enable Test");

   $fdisplay (file_ptr, $time, " ns, Test 3 -- Byte Enable Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 4");

	$display ($time, "ns Finished test 4");





      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;









		pci_ICMD = `TEST5;   //  Offset test



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 5 -- Offset Test");

   $fdisplay (file_ptr, $time, " ns, Test 5 -- Offset Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 5");

	$display ($time, "ns Finished test 5");





      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;





	pci_ICMD = `TEST6;   //  Register test



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 6 -- Register Test");

   $fdisplay (file_ptr, $time, " ns, Test 6 -- Register Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 6");

   $display ($time, "ns Finished test 6");



         $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;





	pci_ICMD = `TEST7;   //  UART test



  @(posedge pci_ICMD_ACK)

	$display ($time, " ns, Test 7 -- UART Test");

   $fdisplay (file_ptr, $time, " ns, Test 7 -- UART Test");

   wait (pci_IBUSY);

   pci_ICMD = `NONE;

   wait (!pci_IBUSY);

   $fdisplay (file_ptr, $time, "ns Finished test 7");

   $display ($time, "ns Finished test 7");



         $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

    	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;

   	tWAIT_4CLKS;





  $display ($time, " ns, Simulation completing normally");

  $fdisplay (file_ptr, $time, " ns, Simulation completing normally");

  $stop(2);

  // syslog.tTERMINATE;

end





//-----------------------------------------------

//  Tasks

//-----------------------------------------------

task tWAIT_4CLKS;

	begin

      @(posedge pci_CLK);

      @(posedge pci_CLK);

      @(posedge pci_CLK);

      @(posedge pci_CLK);

	end

endtask



// This section will stop the simulation if it runs too long

always

	begin

		#500000000

		tWAIT_4CLKS;

		$display ($time, " ns, *****SIMULATION STOPPED DUE TO TIME LIMIT *****");

		$fdisplay (file_ptr, $time, " ns, *****SIMULATION STOPPED DUE TO TIME LIMIT *****");

		$stop(2);

	end





endmodule

