/*----------------------------------------------------------------------------------------------------

--

-- File name	 : write_padder.V

--					 :

-- Title			 : WRITE PADDER

--					 :

-- Purpose		 : PADS the write FIFO with the byte_enables set to 1 (no data) so there is always

--              : enough data for an SDRAM burst

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 23, 2001

--					 :

-- Comments		 :

--					 :

--					 :

--					 : For the Spartan 2 Reference Board

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No 	| Author	 	| Changes Made:							| Mod. Date:

--		v1.0			| da			| First write								| 23/8/01

----------------------------------------------------------------------------------------------------*/



// This is a simple synchronous FIFO.

// It assumes that each burst of data will be 8 units long and that

// each burst will be emptied before a new one is loaded





// This module determined if the write FIFO has a multiple of 8 items of data, and if not

// it will pad it with 8



module WRITE_PADDER (

      CLK,

      RESET,

      BEGIN_WRITE,         // The pci controller is finished

      BEGIN_WRITE_OK,      // The data is correctly padded

      WRITE_COUNT,         // The amount of Data in the write FIFO

      PAD_WRITE           // The instruction to pad the data

      );



input    	CLK;

input    	RESET;

input    	BEGIN_WRITE;

output   	BEGIN_WRITE_OK;

input [2:0] WRITE_COUNT;

output      PAD_WRITE;





//reg         PAD_WRITE;

reg         BEGIN_WRITE_OK;



always @(posedge CLK or posedge RESET)

   begin

   if (RESET)

      BEGIN_WRITE_OK = 1'b0;

   else

      begin

      if (WRITE_COUNT == 3'b000 && BEGIN_WRITE)

         BEGIN_WRITE_OK = 1'b1;

      else

         BEGIN_WRITE_OK = 1'b0;

      end

   end



assign PAD_WRITE = (BEGIN_WRITE && (WRITE_COUNT != 3'b000));





endmodule