#ifndef _SII200_H_

#define _SII200_H_



#include <wdm.h>

#include "ioctrl.h"



// defined types

typedef enum _DEVICE_PNP_STATE {



  NotStarted = 0,         // Not started yet

  Deleted,                // Device has received the REMOVE_DEVICE IRP

  SurpriseRemovePending,  // Device has received the SURPRISE_REMOVE IRP

  Stopped,                // Device has received the STOP_DEVICE IRP

  RemovePending,          // Device has received the QUERY_REMOVE IRP

  FailHardware,           // Fail hardware transfers

  StopPending,            // Device has received the QUERY_STOP IRP

  FailSoftware,           // Fail incoming IRPs

  Started                 // Device has received the START_DEVICE IRP



} DEVICE_PNP_STATE;



typedef struct _DEVICE_EXTENSION  {



  PDEVICE_OBJECT        PhysicalDeviceObject;

  PDEVICE_OBJECT        FunctionDeviceObject;

  PDEVICE_OBJECT        NextStackDeviceObject;



  UNICODE_STRING        InterfaceName;



  DEVICE_PNP_STATE      DevicePnPState;

  DEVICE_PNP_STATE      PreviousPnPState;



  // Memory Information

  ULONG					bar;

  PHYSICAL_ADDRESS      Memory[2];

  PVOID                 MemoryStart[2];

  ULONG                 MemoryOffset[2];

  ULONG                 MemoryLength[2];



} DEVICE_EXTENSION, *PDEVICE_EXTENSION;



// macros

#define INITIALIZE_PNP_STATE(_Data_)    \

        (_Data_)->DevicePnPState =  NotStarted;\

        (_Data_)->PreviousPnPState = NotStarted;



#define SET_NEW_PNP_STATE(_Data_, _state_) \

        (_Data_)->PreviousPnPState =  (_Data_)->DevicePnPState;\

        (_Data_)->DevicePnPState = (_state_);



#define RESTORE_PREVIOUS_PNP_STATE(_Data_)   \

        (_Data_)->DevicePnPState =   (_Data_)->PreviousPnPState;



// forward declarations



// sii200.c

NTSTATUS

DriverEntry (

  IN PDRIVER_OBJECT     DriverObject,

  IN PUNICODE_STRING    RegistryPath

  );



VOID

Unload (

  IN PDRIVER_OBJECT     DriverObject

  );



NTSTATUS

Dispatch (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



// pnp.c

NTSTATUS

AddDevice (

  IN PDRIVER_OBJECT     DriverObject,

  IN PDEVICE_OBJECT     PhysicalDeviceObject

  );



NTSTATUS

Control (

  PDEVICE_OBJECT DeviceObject,

  PIRP Irp

  );



NTSTATUS

DispatchPnP (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



NTSTATUS

DispatchPower (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



VOID

LogMsg(

	IN PDEVICE_OBJECT deviceObject,		//  Device Object

	IN const wchar_t *logMsg			//  Message for event log

	);



NTSTATUS

StartIo(

    IN PDEVICE_OBJECT DeviceObject,

    IN PIRP Irp

    );



#endif // _SII200_H_



#ifdef GUID_DRIVER



// {E471178B-3DA2-455d-AE7F-6894BAA7543F}

static const GUID GuidDriver = 

{ 0xE471178B, 0x3DA2, 0x455d, { 0xAE, 0x7F, 0x68, 0x94, 0xBA, 0xA7, 0x54, 0x3F } };



#endif // GUID_DRIVER



